@@ -1,23 +1,26 @@
 use crate::decrypt::crypto::decrypt_bytes;
+use crate::decrypt::crypto::decrypt_bytes_stream;
+use crate::decrypt::key::get_user_key;
 use crate::file::get_encrypted_file_data;
-use crate::file::get_file_bytes;
 use crate::file::overwrite_check;
 use crate::file::write_bytes_to_file;
 use crate::hashing::hash_data_blake3;
-use crate::prompt::*;
-use crate::structs::*;
 
+use crate::prompt::get_answer;
+use crate::structs::DexiosFile;
 use anyhow::{Context, Ok, Result};
+use std::fs::File;
 
 use std::process::exit;
 use std::time::Instant;
 mod crypto;
+mod key;
 
 pub fn decrypt_file(
     input: &str,
     output: &str,
     keyfile: &str,
-    sha_sum: bool,
+    hash_mode: bool,
     skip: bool,
     bench: bool,
 ) -> Result<()> {
@@ -35,7 +38,7 @@ pub fn decrypt_file(
     let read_duration = read_start_time.elapsed();
     println!("Read {} [took {:.2}s]", input, read_duration.as_secs_f32());
 
-    if sha_sum {
+    if hash_mode {
         let start_time = Instant::now();
         let hash = hash_data_blake3(&data)?;
         let duration = start_time.elapsed();
@@ -56,19 +59,7 @@ pub fn decrypt_file(
         }
     }
 
-    let raw_key = if !keyfile.is_empty() {
-        println!("Reading key from {}", keyfile);
-        get_file_bytes(keyfile)?
-    } else if std::env::var("DEXIOS_KEY").is_ok() {
-        println!("Reading key from DEXIOS_KEY environment variable");
-        std::env::var("DEXIOS_KEY")
-            .context("Unable to read DEXIOS_KEY from environment variable")?
-            .into_bytes()
-    } else {
-        println!("Reading key from stdin");
-        let input = rpassword::prompt_password("Password: ").context("Unable to read password")?;
-        input.as_bytes().to_vec()
-    };
+    let raw_key = get_user_key(keyfile)?;
 
     let decrypt_start_time = Instant::now();
     let decrypted_bytes = decrypt_bytes(data, raw_key)?;
@@ -91,3 +82,34 @@ pub fn decrypt_file(
 
     Ok(())
 }
+
+pub fn decrypt_file_stream(
+    input: &str,
+    output: &str,
+    keyfile: &str,
+    skip: bool,
+    bench: bool,
+) -> Result<()> {
+    if !overwrite_check(output, skip)? {
+        exit(0);
+    }
+
+    let raw_key = get_user_key(keyfile)?;
+
+    let mut input_file = File::open(input).context("Unable to open file")?;
+    let mut output_file = File::create(output).context("Unable to open file")?;
+
+    println!(
+        "Decrypting {} in stream mode (this may take a while)",
+        input
+    );
+    let decrypt_start_time = Instant::now();
+    decrypt_bytes_stream(&mut input_file, &mut output_file, raw_key, bench)?;
+    let decrypt_duration = decrypt_start_time.elapsed();
+    println!(
+        "Decryption successful! [took {:.2}s]",
+        decrypt_duration.as_secs_f32()
+    );
+
+    Ok(())
+}
