@@ -1,21 +1,24 @@
 use crate::encrypt::crypto::encrypt_bytes;
+use crate::encrypt::crypto::encrypt_bytes_stream;
+use crate::encrypt::key::get_user_key;
 use crate::file::get_file_bytes;
 use crate::file::overwrite_check;
 use crate::file::write_encrypted_data_to_file;
 use crate::hashing::hash_data_blake3;
 use anyhow::Context;
 use anyhow::{Ok, Result};
+use std::fs::File;
 use std::process::exit;
 use std::time::Instant;
 
 mod crypto;
-mod password;
+mod key;
 
 pub fn encrypt_file(
     input: &str,
     output: &str,
     keyfile: &str,
-    sha_sum: bool,
+    hash_mode: bool,
     skip: bool,
     bench: bool,
 ) -> Result<()> {
@@ -23,18 +26,9 @@ pub fn encrypt_file(
         exit(0);
     }
 
-    let raw_key = if !keyfile.is_empty() {
-        println!("Reading key from {}", keyfile);
-        get_file_bytes(keyfile)?
-    } else if std::env::var("DEXIOS_KEY").is_ok() {
-        println!("Reading key from DEXIOS_KEY environment variable");
-        std::env::var("DEXIOS_KEY")
-            .context("Unable to read DEXIOS_KEY from environment variable")?
-            .into_bytes()
-    } else {
-        println!("Reading key from stdin");
-        password::get_password_with_validation()?
-    };
+    // add a check for "output file is larger than recommended, would you like to use stream encryption?"
+
+    let raw_key = get_user_key(keyfile)?;
 
     let read_start_time = Instant::now();
     let file_contents = get_file_bytes(input)?;
@@ -60,7 +54,7 @@ pub fn encrypt_file(
         );
     }
 
-    if sha_sum {
+    if hash_mode {
         let hash_start_time = Instant::now();
         let hash = hash_data_blake3(&data)?;
         let hash_duration = hash_start_time.elapsed();
@@ -73,3 +67,34 @@ pub fn encrypt_file(
 
     Ok(())
 }
+
+pub fn encrypt_file_stream(
+    input: &str,
+    output: &str,
+    keyfile: &str,
+    skip: bool,
+    bench: bool,
+) -> Result<()> {
+    if !overwrite_check(output, skip)? {
+        exit(0);
+    }
+
+    let raw_key = get_user_key(keyfile)?;
+
+    let mut input_file = File::open(input).context("Unable to open file")?;
+    let mut output_file = File::create(output).context("Unable to open output file")?;
+
+    println!(
+        "Encrypting {} in stream mode (this may take a while)",
+        input
+    );
+    let encrypt_start_time = Instant::now();
+    encrypt_bytes_stream(&mut input_file, &mut output_file, raw_key, bench)?;
+    let encrypt_duration = encrypt_start_time.elapsed();
+    println!(
+        "Encryption successful! [took {:.2}s]",
+        encrypt_duration.as_secs_f32()
+    );
+
+    Ok(())
+}
