@@ -11,7 +11,7 @@ mod structs;
 
 fn main() -> Result<()> {
     let matches = Command::new("dexios")
-        .version("6.0.2")
+        .version("6.3.2")
         .author("brxken128 <github.com/brxken128>")
         .about("Secure command-line encryption of files.")
         .subcommand_required(true)
@@ -48,11 +48,11 @@ fn main() -> Result<()> {
                         .help("securely erase the input file once complete"),
                 )
                 .arg(
-                    Arg::new("sha")
-                        .short('s')
-                        .long("sha512sum")
+                    Arg::new("hash")
+                        .short('H')
+                        .long("hash")
                         .takes_value(false)
-                        .help("return a sha3-512 hash of the encrypted file"),
+                        .help("return a blake3 hash of the encrypted file"),
                 )
                 .arg(
                     Arg::new("skip")
@@ -67,6 +67,13 @@ fn main() -> Result<()> {
                         .long("benchmark")
                         .takes_value(false)
                         .help("don't write the output file to the disk, to prevent wear on flash storage when benchmarking"),
+                )
+                .arg(
+                    Arg::new("stream")
+                        .short('s')
+                        .long("stream")
+                        .takes_value(false)
+                        .help("use stream encryption (ideal for large files)"),
                 ),
         )
         .subcommand(
@@ -102,11 +109,11 @@ fn main() -> Result<()> {
                         .help("securely erase the input file once complete"),
                 )
                 .arg(
-                    Arg::new("sha")
-                        .short('s')
-                        .long("sha512sum")
+                    Arg::new("hash")
+                        .short('H')
+                        .long("hash")
                         .takes_value(false)
-                        .help("return a sha3-512 hash of the encrypted file"),
+                        .help("return a blake3 hash of the encrypted file"),
                 )
                 .arg(
                     Arg::new("skip")
@@ -121,6 +128,13 @@ fn main() -> Result<()> {
                         .long("benchmark")
                         .takes_value(false)
                         .help("don't write the output file to the disk, to prevent wear on flash storage when benchmarking"),
+                )
+                .arg(
+                    Arg::new("stream")
+                        .short('s')
+                        .long("stream")
+                        .takes_value(false)
+                        .help("use stream decryption (ideal for large files)"),
                 ),
         )
         .get_matches();
@@ -134,18 +148,34 @@ fn main() -> Result<()> {
                     .context("No keyfile/invalid text provided")?;
             }
 
-            let result = encrypt::encrypt_file(
-                sub_matches
-                    .value_of("input")
-                    .context("No input file/invalid text provided")?,
-                sub_matches
-                    .value_of("output")
-                    .context("No output file/invalid text provided")?,
-                keyfile,
-                sub_matches.is_present("sha"),
-                sub_matches.is_present("skip"),
-                sub_matches.is_present("bench"),
-            );
+            let result = if sub_matches.is_present("stream") {
+                // if we're streaming or not
+                encrypt::encrypt_file_stream(
+                    sub_matches
+                        .value_of("input")
+                        .context("No input file/invalid text provided")?,
+                    sub_matches
+                        .value_of("output")
+                        .context("No output file/invalid text provided")?,
+                    keyfile,
+                    sub_matches.is_present("skip"),
+                    sub_matches.is_present("bench"),
+                )
+            } else {
+                encrypt::encrypt_file(
+                    sub_matches
+                        .value_of("input")
+                        .context("No input file/invalid text provided")?,
+                    sub_matches
+                        .value_of("output")
+                        .context("No output file/invalid text provided")?,
+                    keyfile,
+                    sub_matches.is_present("hash"),
+                    sub_matches.is_present("skip"),
+                    sub_matches.is_present("bench"),
+                )
+            };
+
             if result.is_ok() && sub_matches.is_present("erase") {
                 erase::secure_erase(
                     sub_matches
@@ -162,18 +192,33 @@ fn main() -> Result<()> {
                     .context("No keyfile/invalid text provided")?;
             }
 
-            let result = decrypt::decrypt_file(
-                sub_matches
-                    .value_of("input")
-                    .context("No input file/invalid text provided")?,
-                sub_matches
-                    .value_of("output")
-                    .context("No output file/invalid text provided")?,
-                keyfile,
-                sub_matches.is_present("sha"),
-                sub_matches.is_present("skip"),
-                sub_matches.is_present("bench"),
-            );
+            let result = if sub_matches.is_present("stream") {
+                decrypt::decrypt_file_stream(
+                    sub_matches
+                        .value_of("input")
+                        .context("No input file/invalid text provided")?,
+                    sub_matches
+                        .value_of("output")
+                        .context("No output file/invalid text provided")?,
+                    keyfile,
+                    sub_matches.is_present("skip"),
+                    sub_matches.is_present("bench"),
+                )
+            } else {
+                decrypt::decrypt_file(
+                    sub_matches
+                        .value_of("input")
+                        .context("No input file/invalid text provided")?,
+                    sub_matches
+                        .value_of("output")
+                        .context("No output file/invalid text provided")?,
+                    keyfile,
+                    sub_matches.is_present("hash"),
+                    sub_matches.is_present("skip"),
+                    sub_matches.is_present("bench"),
+                )
+            };
+
             if result.is_ok() && sub_matches.is_present("erase") {
                 erase::secure_erase(
                     sub_matches
